/********************************************************* {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* IBM(R) Workplace Forms(tm) Server - Webform Server
*
* (C) Copyright IBM Corp. 2000, 2006
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
********************************************************** {COPYRIGHT-END} **/
package com.PureEdge.pws.sample;

import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.text.DecimalFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * This class contains General utilities for displaying a list of files in HTML.
 */
public class FileUtilities
{
    public static class FormFiles implements FilenameFilter
    {
        public boolean accept(File dir, String name)
		{
		    name = name.toLowerCase();
			if (name.endsWith(".xfd") || name.endsWith(".xfdl") || name.endsWith(".frm"))
				return true;

			return false;
		}
    }


	/**
	 * Delete a file from the filesystem.
	 *
	 * @param filename the name of the file
	 * @param startingDir the starting directory
	 * @param submissionsDir the submissions directory within the starting directory
	 * @throws IOException
	 */
	public static void deleteFile (String filename, String startingDir, String submissionsDir) throws IOException
	{
		String path = startingDir + submissionsDir;

		File file = new File(path + File.separator + filename);

		if (file == null)
			throw new IOException("File " + path + File.separator + filename + " not found.");

		file.delete();
	}

	/**
	 * Retrieves a file from the filesystem. File must be in the starting directory.
	 *
	 * @param dirname directory name that contains the file to be retrieved
	 * @param filename name of the file to retrieve
	 * @param startingDir the starting directory
	 * @return File
	 * @throws IOException
	 */
	public static File getFile(String dirname, String filename, String startingDir) throws IOException
	{
		String path = startingDir;
	    if (dirname.length() > 0)
		    path = path + File.separator + dirname;

		File file = new File(path + File.separator + filename);

		if (file == null)
			throw new IOException("File " + path + File.separator + filename + " not found.");

		return file;
	}

	/**
	 * Sends the contents of a file to the output stream.
	 *
	 * @param file the file to output
	 * @param out the output stream
	 * @throws IOException
	 */
	public static void sendFileToOutputStream(File file, OutputStream out) throws IOException
	{
		FileInputStream fis = new FileInputStream(file);

		byte [] bytes = new byte[32*1024];
		int bytesRead;

		while ((bytesRead = fis.read(bytes, 0, bytes.length)) != -1)
			out.write(bytes, 0, bytesRead);
		fis.close();
	}

	/**
	 * Generates the HTML representing a list of files and directories.
	 *
	 * @param renderCall renderURL String
	 * @param actionCall actionURL String
	 * @param dirname the current directory name to be listed
	 * @param path the current directory path
	 * @param allowDelete allow the deletion of files in this directory
	 * @return HTML representing a list of files
	 * @throws IOException
	 */
	public static String getDirectoryInHTML(String renderCall, String actionCall, String dirname, File path, boolean allowDelete) throws IOException
	{
		StringBuffer html = new StringBuffer();
		File [] files = path.listFiles(new FilenameFilter()
			{
				public boolean accept(File dir, String name)
				{
				    name = name.toLowerCase();
					if (name.endsWith(".xfd") || name.endsWith(".xfdl") || name.endsWith(".frm"))
						return true;

					return false;
				}
			}
		);

		File [] dirs = path.listFiles(new FileFilter()
			{
				public boolean accept(File pathName)
				{
					if (pathName.isDirectory())
						return true;
					return false;
				}
			}
		);

		html.append("\n<table width=\"100%\">");
		html.append("\n<tr>");
		html.append("\n<td></td>");
		html.append("\n<td nowrap><span style=\"font-family: Courier,mono; font-size:10pt;\">Name</span>&nbsp;&nbsp;</td>");
		html.append("\n<td nowrap><span style=\"font-family: Courier,mono; font-size:10pt;\">Last Modified</span>&nbsp;&nbsp;</td>");
		html.append("\n<td align=\"right\" nowrap><span style=\"font-family: Courier,mono; font-size:10pt;\">Size</span>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");
		html.append("\n<td width=\"100%\" nowrap></td>");
		html.append("\n</tr>");
		if (allowDelete)
			html.append("\n<td></td>");
		html.append("\n<tr>");
		if (allowDelete)
			html.append("\n<td colspan=\"7\" width=\"100%\"><hr></td>");
		else
			html.append("\n<td colspan=\"6\" width=\"100%\"><hr></td>");
		html.append("\n</tr>");


		String prevDir = "";
		if (dirname.length() > 0)
		{
			int index = dirname.lastIndexOf("/");
			if (index != -1)
				prevDir = dirname.substring(0, index);

		    html.append("\n<tr>");
			html.append("\n<td nowrap><IMG SRC=\"icons/back.gif\" ALT=\"Directory\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");
			html.append("\n<td nowrap><a style=\"font-family: Courier,mono; font-size:10pt;\" href=\""+renderCall+"?dirname=" + URLEncoder.encode(prevDir) + "\">Parent Directory</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");
			html.append("\n<td nowrap></td>");
			html.append("\n<td nowrap></td>");
			html.append("\n<td nowrap></td>");
			html.append("\n<td nowrap></td>");
			if (allowDelete)
				html.append("\n<td nowrap></td>");
			html.append("\n</tr>");
		}

		if (dirs != null)
		for (int i=0; i < dirs.length; i++)
		{
			html.append("\n<tr>");
			html.append("\n<td nowrap><IMG SRC=\"icons/folder.gif\" ALT=\"Directory\"></td>");
			html.append("\n<td nowrap><a style=\"font-family: Courier,mono; font-size:10pt;\" href=\""+renderCall+"?dirname=" + URLEncoder.encode(dirname +  "/" +  dirs[i].getName()) + "\">" + dirs[i].getName() + "</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");
			html.append("\n<td nowrap><span style=\"font-family: Courier,mono; font-size:10pt;\">" + convertDate(new Date(dirs[i].lastModified())) + "</span>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");
			html.append("\n<td nowrap align=\"right\"><span style=\"font-family: Courier,mono; font-size:10pt;\">DIR</span>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");
			html.append("\n<td nowrap></td>");
			html.append("\n<td nowrap></td>");
			if (allowDelete)
				html.append("\n<td nowrap></td>");
			html.append("\n</tr>");
		}
		if (files != null)
		for (int i=0; i < files.length; i++)
		{
			html.append("\n<tr>");
			html.append("\n<td nowrap><a href=\""+actionCall+"?action=open&mode=xfdl&dirname="+URLEncoder.encode(dirname)+"&filename="+  URLEncoder.encode(files[i].getName()) +"\"><IMG BORDER=\"0\" SRC=\"icons/xfdl.gif\" ALT=\"Force XFDL\"></a>");
			html.append("\n<a href=\""+actionCall+"?action=open&mode=html&dirname="+URLEncoder.encode(dirname)+"&filename="+  URLEncoder.encode(files[i].getName()) +"\"><IMG BORDER=\"0\" SRC=\"icons/pws.gif\" ALT=\"Force HTML\"></a></td>");
			html.append("\n<td nowrap><a title=\"Auto Detect Viewer\" style=\"font-family: Courier,mono; font-size:10pt;\" href=\""+actionCall+"?action=open&dirname=" + URLEncoder.encode(dirname) + "&filename=" + URLEncoder.encode(files[i].getName()) + "\">" + files[i].getName() + "</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");
			html.append("\n<td nowrap><span style=\"font-family: Courier,mono; font-size:10pt;\">" + convertDate(new Date(files[i].lastModified())) + "</span>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");
			html.append("\n<td nowrap align=\"right\"><span style=\"font-family: Courier,mono; font-size:10pt;\">" + convertFileSize(files[i].length()) + "</span>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");
			html.append("\n<td nowrap></td>");
			html.append("\n<td nowrap></td>");
			if (allowDelete)
			{
				html.append("\n<td nowrap><a href=\""+renderCall+"?action=delete&filename="+  URLEncoder.encode(files[i].getName()) +"\"><IMG BORDER=\"0\" SRC=\"icons/delete.gif\" ALT=\"Remove Form\"></a></td>");
			}
			html.append("\n</tr>");
		}

		html.append("\n<tr>");
		if (allowDelete)
			html.append("\n<td colspan=\"7\" width=\"100%\"><hr></td>");
		else
			html.append("\n<td colspan=\"6\" width=\"100%\"><hr></td>");
		html.append("\n</tr>");

		html.append("\n</table>");

		return html.toString();

	}

	/**
	 * Convert a long file size into a more readable format
	 *
	 * @param size size of file as a long
	 *
	 * @return a String containing a more readable size value
	 */
	private static String convertFileSize(long size)
	{
		if (size == 0)
			return ("0");

		Double value = null;
		String suffix = null;

		if (size < 1024)
		{
			value = new Double(size);
			suffix = "B";
		}
		else
		{
			value = new Double (size / 1024 + "." + size % 1024);
			suffix = "K";
		}

		// Format the number

		DecimalFormat df = new DecimalFormat("###,###,###.#");
		return (df.format(value.doubleValue()).toString() + suffix);
	}

	/**
	 * Convert a date into a readable format
	 *
	 * @param date the date to convert
	 *
	 * @return a String containing a formatted date
	 */
	private static String convertDate(Date date)
	{
		String [] monthsAbbr = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul",
								"Aug", "Sep", "Oct", "Nov", "Dec"};

		if (date == null)
			return("");

		Calendar cal = Calendar.getInstance();

		cal.setTime(date);

		/* Get the date integers */

		int dayOfMonth = cal.get(java.util.Calendar.DATE);
		int month = cal.get(java.util.Calendar.MONTH);
		int year = cal.get(java.util.Calendar.YEAR);

		int hour = cal.get(Calendar.HOUR_OF_DAY);
		int minute = cal.get(Calendar.MINUTE);

		/* Convert to strings */

		String currentMonthAbbr = monthsAbbr[month];
		String currentDay = java.lang.Integer.toString(dayOfMonth);
		String currentYear = java.lang.Integer.toString(year);

		if (dayOfMonth < 10)
			currentDay = "0" + currentDay;

		String currentHour = Integer.toString(hour);
		String currentMinute = Integer.toString(minute);

		if (hour < 10)
			currentHour = "0" + currentHour;

		if (minute < 10)
			currentMinute = "0" + currentMinute;

		return (currentDay + "-" + currentMonthAbbr + "-" + currentYear + " " +
				currentHour + ":" + currentMinute);
	}

	/**
	 * Copies a directory of files from a source dir to destination dir.
	 *
	 * @param sourceDir The source directory
	 * @param destinationDir The destination directory
	 * @throws Exception
	 */
    public static void copyFiles(File sourceDir, File destinationDir) throws Exception
    {
        if (sourceDir != null && sourceDir.exists())
        {
            FormFiles formFiles = new FormFiles();
            File [] files = sourceDir.listFiles(formFiles);
            if (files != null)
                for (int i=0; i < files.length; i++)
                    copyFile(files[i], new File(destinationDir, files[i].getName()));
        }
    }

	/**
	 * Copies a file to a destination file.
	 *
	 * @param sourceDir The source file
	 * @param destinationDir The destination file
	 * @throws Exception
	 */
    public static void copyFile(File source, File dest) throws Exception
	{
		FileInputStream fis  = new FileInputStream(source);
		FileOutputStream fos = new FileOutputStream(dest);
		byte[] buf = new byte[32*1024];
		int i = 0;
		while((i=fis.read(buf))!=-1)
		{
		  fos.write(buf, 0, i);
		}
		fis.close();
		fos.close();
	}

}
