/********************************************************* {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* IBM(R) Workplace Forms(tm) Server - Webform Server
*
* (C) Copyright IBM Corp. 2000, 2006
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
********************************************************** {COPYRIGHT-END} **/
package com.PureEdge.pws.sample.servlet;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URLEncoder;
import java.rmi.server.UID;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.PureEdge.DTK;
import com.PureEdge.IFSSingleton;
import com.PureEdge.error.UWIException;
import com.PureEdge.pws.sample.FileUtilities;
import com.PureEdge.servlet.PureEdgeServlet;
import com.PureEdge.xfdl.FormNodeP;
import com.PureEdge.xfdl.XFDL;

public class FormHandler extends PureEdgeServlet
{
	private String startingDir = null;
	private String submissionsDir = null;

	/**
	 * Constructor of the object.
	 */
	public FormHandler()
	{
		super();
	}

	/**
	 * Initialization of the servlet. <br>
	 * For this sample, we need to determine the root level of
	 *
	 * @throws ServletException if an error occures
	 */
	public void init() throws ServletException
	{
		// you must call the super.init() when extending the PureEdgeServlet
		super.init();
		ServletContext context = this.getServletContext();
		// here we are reading a few context parameters from the web.xml file
		this.startingDir = context.getInitParameter("startingDirectory");
		this.submissionsDir = context.getInitParameter("submissionsDirectory");

		// if the context parameters don't exist or are blank, set defaults.
		if (this.startingDir == null)
			this.startingDir = "";

		if (this.startingDir.length() == 0)
			this.startingDir = context.getRealPath("/") + File.separator + "ServletSampleForms";

		if (this.submissionsDir == null)
			this.submissionsDir = "";

		if (this.submissionsDir.length() == 0)
			this.submissionsDir = File.separator + "submissions";

		// create submissions directory
		File submissionsFile = new File(this.startingDir + this.submissionsDir);
		if (submissionsFile.exists() == false) // then we need to create it
			submissionsFile.mkdirs(); // this in turn will also create the starting directory if necessary.

		// initialize the PureEdge API
		try
		{
			DTK.initialize("FormHandler", "1.0.0", "@CORE_MAJOR_VERSION@.@CORE_MINOR_VERSION@.@CORE_MAINT_VERSION@");
		}
		catch (UWIException e)
		{
			throw new ServletException("Could not initialize the PureEdge API, reason: /n" + e.getStackTraceString());
		}
	}

	/**
	 * Destruction of the servlet. <br>
	 */
	public void destroy()
	{
		super.destroy();
	}

	/**
	 * The doGet method of the servlet. <br>
	 *
	 * This method is called when a form has its tag value method equals to get.
	 *
	 * @param request the request send by the client to the server
	 * @param response the response send by the server to the client
	 * @throws ServletException if an error occurred
	 * @throws IOException if an error occurred
	 */
	public void doGet(HttpServletRequest request, HttpServletResponse response)	throws ServletException, IOException
	{
	    FormNodeP theForm = null;
		String mode = request.getParameter("mode");
		if (mode != null)
		{
			if (mode.equalsIgnoreCase("xfdl"))
				this.useXFDL(response, true);
			else if (mode.equalsIgnoreCase("html"))
				this.useHTML(response, true);
			else
				throw new ServletException("mode parameter can only be 'xfdl' or 'html'");
		}

		String dirname = request.getParameter("dirname");
		if (dirname == null)
			dirname= "";

		String filename = request.getParameter("filename");

		File testFile = new File(this.startingDir + dirname + filename);
		if (testFile.getCanonicalPath().startsWith(new File(this.startingDir).getCanonicalPath()) == false)
			throw new ServletException("Dir must be within the starting directory.");

		String action = request.getParameter("action");
		if (action != null)
		{
			if (action.equalsIgnoreCase("open"))
			{
			    try
			    {
					File file = FileUtilities.getFile(dirname, filename, this.startingDir);
					theForm = prepopForm(file, request, dirname, filename);

					// must set the filename so that if the user hits the save button, it knows what to call it by default.
					this.setFilename(response, filename);
					response.setContentType("application/vnd.xfdl");
					theForm.writeForm(response.getOutputStream(), null, 0);
			    }
			    catch (UWIException e)
		        {
		            throw new ServletException("Could not write out form, reason:\n" + e.getStackTraceString());
		        }
			    finally
		        {
		            if (theForm != null)
		            {
		                try
		                {
		                    theForm.destroy();
		                }
		                catch (UWIException e)
		                {
		                	throw new ServletException("Could not destroy form, reason:\n" + e.getStackTraceString());
		                }
		            }
		        }

			}
			else if (action.equalsIgnoreCase("delete"))
			{
				FileUtilities.deleteFile(filename, this.startingDir, this.submissionsDir);
				response.sendRedirect("viewForms.jsp?dirname="+ URLEncoder.encode(this.submissionsDir));
			}
			else
				throw new ServletException("action parameter can only be 'open' or 'delete'");
		}
		else
			response.sendRedirect("viewForms.jsp");

	}

    /**
	 * The doPost method of the servlet. <br>
	 *
	 * This method is called when a form has its tag value method equals to post.
	 *
	 * @param request the request send by the client to the server
	 * @param response the response send by the server to the client
	 * @throws ServletException if an error occurred
	 * @throws IOException if an error occurred
	 */
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
	{
		String dirname = request.getParameter("dirname");
		String filename = request.getParameter("filename");
		FormNodeP theForm = null;

		if (dirname == null)
			dirname= "";

		//if a filename was not provided, then use the currently known filename
		if (filename == null)
			filename= this.getFilename(request);

        try
        {
            XFDL theXFDL = IFSSingleton.getXFDL();
            theForm = theXFDL.readForm(request.getInputStream(), XFDL.UFL_SERVER_SPEED_FLAGS);
            if (filename.lastIndexOf("_pws_") == -1)
            {
	            // create unique id for this form
    	        UID uid = new UID();
        	    filename = uid.toString().replace(':','_') + "_pws_" + filename;
        	}
            theForm.writeForm(this.startingDir + this.submissionsDir + File.separator + filename, null, 0);
        }
        catch (UWIException e)
        {
        	throw new ServletException("Could not save submitted form, reason:\n" + e.getStackTraceString());
        }
        finally
        {
            if (theForm != null)
            {
                try
                {
                    theForm.destroy();
                }
                catch (UWIException e)
                {
                	throw new ServletException("Could not destroy form, reason:\n" + e.getStackTraceString());
                }
            }
        }

        response.sendRedirect("viewForms.jsp?dirname="+ URLEncoder.encode(dirname));
	}

	private FormNodeP prepopForm(File file, HttpServletRequest request, String dirname, String filename) throws ServletException, IOException
	{
		// first get inputStream to file (the form)
		FileInputStream fis = null;
		try
		{
			fis = new FileInputStream(file);
		}
		catch (FileNotFoundException e)
		{
			throw new ServletException("File '" + file.getName() + "' not found.");
		}

		// then using that inputStream read the form into the PureEdge API
		FormNodeP theForm = null;
		try
		{
		    // in order to read a form into the PureEdge API, you must obtain an XFDL object
		    // you do this by calling IFSSingleton,getXFDL() like below:
			XFDL theXFDL = IFSSingleton.getXFDL();

			// now that you have an XFDL object, you can ask it to read in a form using the readForm method.
			theForm = theXFDL.readForm(fis, XFDL.UFL_SERVER_SPEED_FLAGS);

			// now see if this form has a global.global.custom:url option, and if it does set it to hit this servlet on this server
			FormNodeP urlNode = theForm.dereferenceEx(null, "global.global.custom:url", 0, FormNodeP.UFL_OPTION_REFERENCE, null);

			// if urlNode is null, then that open does not exist in the form, and this form should be left alone
			if (urlNode != null)
			{
				// first create the url that will submit to this server
				String fullURL = null;

				// use relative urls
				String uri = request.getRequestURI();
				fullURL = uri.substring(uri.lastIndexOf("/") + 1);
				fullURL = fullURL + "?dirname=" + URLEncoder.encode(dirname) + "&filename=" + URLEncoder.encode(filename);

				// then save that value in the form
				// all the submit buttons in the form should computer their <url> open referencing: global.global.custom:url
				urlNode.setLiteral(fullURL);
			}
		}
		catch (UWIException e)
		{
			throw new ServletException("Error Loading File '" + file.getName() + "', reason:/n" + e.getStackTraceString());
		}
		finally
		{
		    fis.close();
		}
		return theForm;
	}
}
