/********************************************************* {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* IBM(R) Workplace Forms(tm) Server - Webform Server
*
* (C) Copyright IBM Corp. 2000, 2006
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
********************************************************** {COPYRIGHT-END} **/
package com.PureEdge.pws.sample.portlet.jsr;

import java.io.File;
import java.io.FileFilter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.Calendar;
import java.util.Date;

import javax.portlet.PortletURL;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

/**
 * This class contains General utilities for displaying a list of files in HTML.
 */
public class FileUtilities
{

	/**
	 * Delete a file from the filesystem.
	 *
	 * @param filename the name of the file
	 * @param startingDir the starting directory
	 * @param submissionsDir the submissions directory within the starting directory
	 * @throws IOException
	 */
	public static void deleteFile (String filename, String startingDir, String submissionsDir) throws IOException
	{
		String path = startingDir + submissionsDir;

		File file = new File(path + File.separator + filename);

		if (file == null)
			throw new IOException("File " + path + File.separator + filename + " not found.");

		file.delete();
	}

	/**
	 * Generates the HTML representing a list of files and directories.
	 *
	 * @param request the Portlet RenderRequest object
	 * @param response the Portlet RenderResponse object
	 * @param dirname the current directory name to be listed
	 * @param path the current directory path
	 * @param allowDelete allow the deletion of files in this directory
	 * @return HTML representing a list of files
	 * @throws IOException
	 */
	public static String getDirectoryInHTML(RenderRequest request, RenderResponse response, String dirname, File path,
			boolean allowDelete) throws IOException
	{
		StringBuffer html = new StringBuffer();
		File [] files = path.listFiles(new FilenameFilter()
			{
				public boolean accept(File dir, String name)
				{
				    name = name.toLowerCase();
					if (name.endsWith(".xfd") || name.endsWith(".xfdl") || name.endsWith(".frm"))
						return true;

					return false;
				}
			}
		);

		File [] dirs = path.listFiles(new FileFilter()
			{
				public boolean accept(File pathName)
				{
					if (pathName.isDirectory())
						return true;
					return false;
				}
			}
		);

		// Output a heading

		html.append("\n<table width=\"100%\">");
		html.append("\n<tr>");
		html.append("\n<td></td>");
		html.append("\n<td nowrap><span style=\"font-family: Courier,mono; font-size:10pt;\">Name</span>&nbsp;&nbsp;</td>");
		html.append("\n<td nowrap><span style=\"font-family: Courier,mono; font-size:10pt;\">Last Modified</span>&nbsp;&nbsp;</td>");
		html.append("\n<td align=\"right\" nowrap><span style=\"font-family: Courier,mono; font-size:10pt;\">Size</span>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");
		html.append("\n<td width=\"100%\" nowrap></td>");
		html.append("\n</tr>");

		html.append("\n<tr>");
		html.append("\n<td colspan=\"5\" width=\"100%\"><hr></td>");
		html.append("\n</tr>");

		// Add a previous dir link (if not in root dir)
		
		if (dirname.length() > 0)
		{
		    PortletURL prevDirURL = response.createActionURL();

			int index = 0;
			if ((index = dirname.lastIndexOf("/")) != -1)
			{
				
				if (index > 1)
					prevDirURL.setParameter("dirname", dirname.substring(0, index));
			}

		    html.append("\n<tr>");
			html.append("\n<td nowrap><IMG SRC=\"" + request.getContextPath() + "/icons/back.gif\" ALT=\"Directory\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");
			html.append("\n<td nowrap><a style=\"font-family: Courier,mono; font-size:10pt;\" href=\""+ prevDirURL.toString() + "\">Parent Directory</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");
			html.append("\n<td nowrap></td>");
			html.append("\n<td nowrap></td>");
			html.append("\n<td nowrap></td>");
			html.append("\n</tr>");
		}

		// Generate a link for each directory

		if (dirs != null)
		{
			for (int i=0; i < dirs.length; i++)
			{
			    PortletURL dirURL = response.createActionURL();
			    dirURL.setParameter("dirname", dirname +  "/" +  dirs[i].getName());

				html.append("\n<tr>");
				html.append("\n<td nowrap><IMG SRC=\"" + request.getContextPath() + "/icons/folder.gif\" ALT=\"Directory\"></td>");
				html.append("\n<td nowrap><a style=\"font-family: Courier,mono; font-size:10pt;\" href=\""+dirURL.toString()+"\">" + dirs[i].getName() + "</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");
				html.append("\n<td nowrap><span style=\"font-family: Courier,mono; font-size:10pt;\">" + convertDate(new Date(dirs[i].lastModified())) + "</span>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");
				html.append("\n<td nowrap align=\"right\"><span style=\"font-family: Courier,mono; font-size:10pt;\">DIR</span>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");
				html.append("\n<td nowrap></td>");
				html.append("\n</tr>");
			}
		}

		// Generate a link for each file

		if (files != null)
		{
			
			for (int i=0; i < files.length; i++)
			{
				PortletURL returnURL = response.createRenderURL();

				PortletURL fileURL = response.createActionURL();
				if (!dirname.equals(""))
				   fileURL.setParameter("dirname", dirname);
				fileURL.setParameter("filename", files[i].getName());
				fileURL.setParameter("portletURL", returnURL.toString());

				PortletURL useViewerFileURL = response.createActionURL();
				if (!dirname.equals(""))
				   useViewerFileURL.setParameter("dirname", dirname);
				useViewerFileURL.setParameter("filename", files[i].getName());
				useViewerFileURL.setParameter("mode", "viewer");
				useViewerFileURL.setParameter("portletURL", returnURL.toString());

				PortletURL usePWSFileURL = response.createActionURL();
				if (!dirname.equals(""))
				  usePWSFileURL.setParameter("dirname", dirname);
				usePWSFileURL.setParameter("filename", files[i].getName());
				usePWSFileURL.setParameter("mode", "pws");
				usePWSFileURL.setParameter("portletURL", returnURL.toString());

				html.append("\n<tr>");
				html.append("\n<td nowrap><a href=\"" + useViewerFileURL.toString() + "\"><IMG BORDER=\"0\" SRC=\"" + request.getContextPath() + "/icons/xfdl.gif\" ALT=\"Force XFDL\"></a>");
				html.append("<a href=\"" + usePWSFileURL.toString() + "\"><IMG BORDER=\"0\" SRC=\"" + request.getContextPath() + "/icons/pws.gif\" ALT=\"Force HTML\"></a></td>");

				html.append("\n<td nowrap><a title=\"Auto Detect Viewer\" style=\"font-family: Courier,mono; font-size:10pt;\" href=\"" + fileURL.toString() + "\">" + files[i].getName() + "</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");
				html.append("\n<td nowrap><span style=\"font-family: Courier,mono; font-size:10pt;\">" + convertDate(new Date(files[i].lastModified())) + "</span>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");
				html.append("\n<td nowrap align=\"right\"><span style=\"font-family: Courier,mono; font-size:10pt;\">" + convertFileSize(files[i].length()) + "</span>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");

				html.append("\n<td width=\"100%\" align=\"left\" nowrap>");
				if (allowDelete)
				{
					PortletURL delURL = response.createActionURL();
					delURL.setParameter("action", "delete");
					if (!dirname.equals(""))
					   delURL.setParameter("dirname", dirname);
					delURL.setParameter("filename", files[i].getName());

					html.append("<a href=\""+delURL.toString()+"\"><IMG BORDER=\"0\" SRC=\"" + request.getContextPath() + "/icons/delete.gif\" ALT=\"Remove Form\"></a>");
				}
				html.append("</td>");

				html.append("\n</tr>");
			}
		}

		html.append("\n<tr>");
		html.append("\n<td colspan=\"5\" width=\"100%\"><hr></td>");
		html.append("\n</tr>");

		html.append("\n</table>");

		return html.toString();

	}

	/**
	 * Convert a long file size into a more readable format
	 *
	 * @param size size of file as a long
	 *
	 * @return a String containing a more readable size value
	 */
	private static String convertFileSize(long size)
	{
		if (size == 0)
			return ("0");

		Double value = null;
		String suffix = null;

		if (size < 1024)
		{
			value = new Double(size);
			suffix = "B";
		}
		else
		{
			value = new Double (size / 1024 + "." + size % 1024);
			suffix = "K";
		}

		// Format the number

		DecimalFormat df = new DecimalFormat("###,###,###.#");
		return (df.format(value.doubleValue()).toString() + suffix);
	}

	/**
	 * Convert a date into a readable format
	 *
	 * @param date the date to convert
	 *
	 * @return a String containing a formatted date
	 */
	private static String convertDate(Date date)
	{
		String [] monthsAbbr = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul",
								"Aug", "Sep", "Oct", "Nov", "Dec"};

		if (date == null)
			return("");

		Calendar cal = Calendar.getInstance();

		cal.setTime(date);

		// Get the date integers

		int dayOfMonth = cal.get(java.util.Calendar.DATE);
		int month = cal.get(java.util.Calendar.MONTH);
		int year = cal.get(java.util.Calendar.YEAR);

		int hour = cal.get(Calendar.HOUR_OF_DAY);
		int minute = cal.get(Calendar.MINUTE);

		// Convert to strings

		String currentMonthAbbr = monthsAbbr[month];
		String currentDay = java.lang.Integer.toString(dayOfMonth);
		String currentYear = java.lang.Integer.toString(year);

		if (dayOfMonth < 10)
			currentDay = "0" + currentDay;

		String currentHour = Integer.toString(hour);
		String currentMinute = Integer.toString(minute);

		if (hour < 10)
			currentHour = "0" + currentHour;

		if (minute < 10)
			currentMinute = "0" + currentMinute;

		return (currentDay + "-" + currentMonthAbbr + "-" + currentYear + " " +
				currentHour + ":" + currentMinute);
	}
}
