/********************************************************* {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* IBM(R) Workplace Forms(tm) Server - Webform Server
*
* (C) Copyright IBM Corp. 2000, 2006
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
********************************************************** {COPYRIGHT-END} **/
package com.PureEdge.pws.sample.portlet.jsr;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.rmi.server.UID;
import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletConfig;
import javax.portlet.PortletContext;
import javax.portlet.PortletException;
import javax.portlet.PortletRequest;
import javax.portlet.PortletRequestDispatcher;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import com.PureEdge.DTK;
import com.PureEdge.IFSSingleton;
import com.PureEdge.error.UWIException;
import com.PureEdge.portlet.jsr.PureEdgePortlet;
import com.PureEdge.xfdl.FormNodeP;
import com.PureEdge.xfdl.XFDL;


public class FormViewPortlet extends PureEdgePortlet
{
	public static final String	SESSION_BEAN = "FormViewSessionBean";

	private PortletContext context;

	/**
	 * Initialize the Portlet
	 *
	 * @param config the PortletConfig object
	 */
	public void init(PortletConfig config) throws PortletException
	{
		super.init(config);
		context = config.getPortletContext();

		// Initialize the PureEdge API

		try
		{
			DTK.initialize("FormViewPortlet", "1.0.0", "@CORE_MAJOR_VERSION@.@CORE_MINOR_VERSION@.@CORE_MAINT_VERSION@");
		}
		catch (UWIException e)
		{
			throw new PortletException("Could not initialize the PureEdge API, reason: /n" + e.getStackTraceString());
		}
	}

	/**
	 * This method is implemented to read any parameters stored in the global application
	 * session. We must get these values before the PureEdgePortlet.doView() is executed so
	 * that we can reset the PureEdgePortlet in order to read in a new form.
	 *
	 * @param request the RenderRequest object
	 * @param response the RenderResponse object
	 *
	 * @throws PortletException
	 * @throws IOException
	 */
	public void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException
	{
		FormViewSessionBean sessionBean = this.getSessionBean(request);

		// Reset the portlet if a new form has been selected

		String currFilename = sessionBean.getFilename();
		String currDirectory = sessionBean.getDirectory();

		String sessFilename = (String) ((RenderRequest)request).getPortletSession().getAttribute("PE_XFDL_Filename", PortletSession.APPLICATION_SCOPE);
		String sessDirectory = (String) ((RenderRequest)request).getPortletSession().getAttribute("PE_XFDL_Directory", PortletSession.APPLICATION_SCOPE);
		String sessMode = (String) ((RenderRequest)request).getPortletSession().getAttribute("PE_XFDL_Mode", PortletSession.APPLICATION_SCOPE);

		if (sessFilename != null)
		{
			sessionBean.setFilename(sessFilename);
			((RenderRequest)request).getPortletSession().setAttribute("PE_XFDL_Filename", null, PortletSession.APPLICATION_SCOPE);

			// If we have a new filename then we must set the directory name
			//	even if it's null

			sessionBean.setDirectory(sessDirectory);
			((RenderRequest)request).getPortletSession().setAttribute("PE_XFDL_Directory", null, PortletSession.APPLICATION_SCOPE);

			if (sessMode != null)
			{
				if (sessMode.equals("viewer"))
					super.useXFDL(request, true);
				else if (sessMode.equals("pws"))
					super.useHTML(request, true);
			}
			else
			{
				super.useHTML(request, false);
				super.useXFDL(request, false);
			}

			super.resetPortlet(request);
		}

		if ((currFilename != null && sessFilename != null) && (currFilename.equals(sessFilename) == false))
			super.resetPortlet(request);
		else if ((currDirectory != null && sessDirectory != null) && (currDirectory.equals(sessDirectory) == false))
			super.resetPortlet(request);

		super.doView(request, response);
	}


	public void doViewEx(RenderRequest request, RenderResponse response) throws PortletException, IOException
	{
		FormViewSessionBean sessionBean = this.getSessionBean(request);

		// Determine the Samples directory

		String startingDir = request.getPreferences().getValue("startingDirectory", null);

		if (startingDir == null)
			startingDir = this.getPortletContext().getRealPath("/") + File.separator + "SampleForms";

		String dirname = sessionBean.getDirectory();
		String filename = sessionBean.getFilename();

		File sampleDirFile = null;

		if (dirname != null)
		    sampleDirFile = new File(startingDir + File.separator + dirname);
		else
		    sampleDirFile = new File(startingDir);

		if ((sampleDirFile == null) || (sampleDirFile.exists() == false))
			throw new PortletException("Directory not found: " + sampleDirFile.getCanonicalPath());

		if ((sessionBean.isSubmitted() == false) && (filename != null))
		{
		    String path = startingDir;
		    if (dirname != null)
			    path = path + File.separator + dirname;

			// A filename was requested so get the file from the sample
			//    directory and return the file

			File file = new File(path + File.separator + filename);

			if ((file == null) || (file.exists() == false))
				throw new PortletException("File not found: " + path + File.separator + filename);

			// Check the canonical path to make sure we are still retrieving
			//	  a file from within the sample directory (security check)

			if (file.getCanonicalPath().startsWith(sampleDirFile.getCanonicalPath()) == false)
				throw new PortletException("File must be retrieved from within the sample file directory.");

			// Write the xfdl file to the servletOuputStream

			setFilename(request, filename);
			response.setContentType("application/vnd.xfdl");

			FileInputStream fos = new FileInputStream(file);

			byte [] bytes = new byte[32000];
			int bytesRead;

			while ((bytesRead = fos.read(bytes, 0, bytes.length)) != -1)
				response.getPortletOutputStream().write(bytes, 0, bytesRead);

			fos.close();
		}
		else
		{
			String jspName = null;

			if (sessionBean.isSubmitted())
			{
				sessionBean.setSubmitted(false);
				jspName = getPortletConfig().getInitParameter("jspSubmit");
			}
			else
				jspName = getPortletConfig().getInitParameter("jspView");

			response.setContentType("text/html");
			PortletRequestDispatcher rd = getPortletContext().getRequestDispatcher(jspName);
			rd.include(request, response);
		}
	}

	public void processActionEx(ActionRequest request, ActionResponse response) throws PortletException, IOException
	{
		FormViewSessionBean sessionBean = this.getSessionBean(request);

		String startingDir = request.getPreferences().getValue("startingDirectory", null);
		String submissionsDir = request.getPreferences().getValue("submissionsDirectory", null);

		if (startingDir == null)
			startingDir = this.getPortletContext().getRealPath("/") + File.separator + "SampleForms";

		if (submissionsDir == null)
			submissionsDir = File.separator + "submissions";

		// Create the submissions directory if it doesn't exist.

		File submissionsFile = new File(startingDir + submissionsDir);
		if (submissionsFile.exists() == false)
			submissionsFile.mkdirs();

		String filename = sessionBean.getFilename();

		FormNodeP theForm = null;
        try
        {
            XFDL theXFDL = IFSSingleton.getXFDL();

            theForm = theXFDL.readForm(request.getPortletInputStream(), XFDL.UFL_SERVER_SPEED_FLAGS);

            if (filename.lastIndexOf("_pws_") == -1)
            {
            	// create unique id for this form

            	UID uid = new UID();
        	    filename = uid.toString().replace(':','_') + "_pws_" + filename;
            }

            theForm.writeForm(startingDir + submissionsDir + File.separator + filename, null, 0);

            // Reset everything and set the submitted flag in the session

            super.resetPortlet(request);
            sessionBean.setSubmitted(true);
            sessionBean.setDirectory(null);
            sessionBean.setFilename(null);
        }
        catch (UWIException e)
        {
        	throw new PortletException("Could not save submitted form, reason:\n" + e.getStackTraceString());
        }
        finally
        {
            if (theForm != null)
            {
                try
                {
                    theForm.destroy();
                }
                catch (UWIException e)
                {
                	throw new PortletException("Could not destroy form, reason:\n" + e.getStackTraceString());
                }
            }
        }
	}

	/**
	 * Gets the sessionBean or creates one if one hasn't been created already
	 *
	 * @param request the PortletRequest
	 *
	 * @return the FormViewSessionBean
	 */
	private FormViewSessionBean getSessionBean(PortletRequest request)
	{
		PortletSession session =  request.getPortletSession();

		FormViewSessionBean sessionBean = (FormViewSessionBean) session.getAttribute(FormViewPortlet.SESSION_BEAN);

		if (sessionBean == null)
		{
			sessionBean = new FormViewSessionBean();
			session.setAttribute(FormViewPortlet.SESSION_BEAN, sessionBean);
		}

		return sessionBean;
	}

}
