/********************************************************* {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* IBM(R) Workplace Forms(tm) Server - Webform Server
*
* (C) Copyright IBM Corp. 2000, 2006
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
********************************************************** {COPYRIGHT-END} **/
package com.PureEdge.pws.sample.portlet.jsr;

import java.io.IOException;
import java.io.File;
import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletConfig;
import javax.portlet.PortletContext;
import javax.portlet.PortletException;
import javax.portlet.PortletRequest;
import javax.portlet.PortletRequestDispatcher;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

/**
 * This class represents a portlet displaying the any forms
 * 	contained in a sample directory.
 */
public class FormsListPortlet extends GenericPortlet
{
	public static final String	SESSION_BEAN = "FormsListSessionBean";

	private PortletContext context;

	/**
	 * Initialize the Portlet
	 *
	 * @param config the PortletConfig object
	 */
	public void init(PortletConfig config) throws PortletException
	{
		super.init(config);
		context = config.getPortletContext();
	}


	public void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException
	{
		// Create the session bean

		FormsListSessionBean sessionBean = this.getSessionBean(request);

		// Include the JSP

		String jspName = getPortletConfig().getInitParameter("jspView");
		PortletRequestDispatcher rd = getPortletContext().getRequestDispatcher(jspName);
		rd.include(request, response);
	}

	public void processAction(ActionRequest request, ActionResponse response) throws PortletException, IOException
	{
		FormsListSessionBean sessionBean = this.getSessionBean(request);

		String filename = request.getParameter("filename");
		String directory = request.getParameter("dirname");
		String action = request.getParameter("action");
		String mode = request.getParameter("mode");

		if ((filename != null) && (action == null))
		{
			// A file was chosen so set the info into the global application session for
			//	the other portlet to consume.

			request.getPortletSession().setAttribute("PE_XFDL_Filename", filename, PortletSession.APPLICATION_SCOPE);
			request.getPortletSession().setAttribute("PE_XFDL_Directory", directory, PortletSession.APPLICATION_SCOPE);
			request.getPortletSession().setAttribute("PE_XFDL_Mode", mode, PortletSession.APPLICATION_SCOPE);
		}
		else
		{
			sessionBean.setFilename(filename);
			sessionBean.setDirectory(directory);

			if ((action != null) && (action.equals("deleteConfirmed")))
			{
				// Delete the file that was selected.

				String startingDir = request.getPreferences().getValue("startingDirectory", null);
				String submissionsDir = request.getPreferences().getValue("submissionsDirectory", null);

				if (startingDir == null)
					startingDir = this.getPortletContext().getRealPath("/") + File.separator + "SampleForms";

				if (submissionsDir == null)
					submissionsDir = File.separator + "submissions";

				FileUtilities.deleteFile(filename, startingDir, submissionsDir);

				// Reset the action.

				sessionBean.setAction(null);
			}
			else
			{
				// Some other action was preformed.

				sessionBean.setAction(action);
			}
		}
	}

	/**
	 * Gets the sessionBean or creates one if one hasn't been created already
	 *
	 * @param request the PortletRequest
	 *
	 * @return the FormListSessionBean
	 */
	private FormsListSessionBean getSessionBean(PortletRequest request)
	{
		PortletSession session =  request.getPortletSession();

		FormsListSessionBean sessionBean = (FormsListSessionBean) session.getAttribute(FormsListPortlet.SESSION_BEAN);

		if (sessionBean == null)
		{
			sessionBean = new FormsListSessionBean();
			session.setAttribute(FormsListPortlet.SESSION_BEAN, sessionBean);
		}

		return sessionBean;
	}

}
